"""Tactus Standard Library.

The standard library consists of .tac files in the tac/ subdirectory.
These are loaded via Lua's require() function:

    local done = require("tactus.tools.done")
    local log = require("tactus.tools.log")

See tactus/stdlib/tac/ for available modules.
"""
